from typing import Tuple, Optional

import many_abis as ma
import requests
from web3 import Web3
from web3 import exceptions as web3exceptions


def create_dex_router(chain: str, dex_name: str, client: Optional[Tuple[str, Web3]] = None):
    if client is None:
        rpc = ma.get_chain(chain).rpcs.all[0]
        client = Web3(Web3.HTTPProvider(rpc))
    elif isinstance(client, str):
        if client.startswith('http'):
            client = Web3(Web3.HTTPProvider(client))
        elif client.startswith('ws'):
            client = Web3(Web3.WebsocketProvider(client))
        else:
            raise NotImplementedError
    elif not isinstance(client, Web3):
        raise NotImplementedError

    dex = ma.get(chain, dex_name)
    return client.eth.contract(dex.router_address, abi=dex.router_abi)


def get_abi_from_address(address: str, api_key: str, chain_api: str):
    '''
    address: Address want to get abi
    api_key: Explorer API Key
    chain_api: api format for get abi, format like this,
        'https://xxx.xxxscan.xxx/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    '''
    try:
        query = chain_api.format(contract_address=address, api_key=api_key)
        session = requests.Session()
        headers = {
            'User-Agent': 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3)'
        }
        result = session.get(query, headers=headers)
        session.close()
        result = eval(result.content.decode())
        if result['status'] == '1' and result['message'] == 'OK':
            return result['result']
        else:
            return None
    except Exception as e:
        print(e)
        return None


def get_factory_from_router(router_address: str, api_key: str, chain_api: str, rpc: str) -> Tuple[str, dict]:
    web3 = Web3(Web3.HTTPProvider(rpc))
    router_address = web3.toChecksumAddress(router_address)
    if router_abi := get_abi_from_address(router_address, api_key, chain_api):
        try:
            router_contract = web3.eth.contract(address=router_address, abi=router_abi)
            factory_address = router_contract.functions.factory().call()
            factory_abi = get_abi_from_address(factory_address, api_key, chain_api)
            return factory_address, factory_abi
        except web3exceptions as e:
            print(f"Something wrong with web3, {e}")
        except Exception as e:
            print(e)



from ._base import *

WFTM = BaseWETH(
    name='Wrapped Fantom Token',
    symbol='WFTM',
    address='0x21be370D5312f44cB42ce377BC9b8a0cEF1A4C83',
)

STABLE_COINS = BaseCoin(
    {
        'USDC': '0x04068DA6C83AFCFA0e13ba15A6696662335D5B75',
        'DAI': '0x8D11eC38a3EB5E956B052f67Da8Bdc9bef8Abf3E',
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/fantom/{lp_address}',
        'dexscreener': 'https://dexscreener.com/fantom/{lp_address}',
        'ChartEX': 'https://chartex.pro/dashboard',
        'Sell/Buy On SpookySwap': 'https://spookyswap.finance/swap',
        'Sell/Buy On SpiritSwap': 'https://swap.spiritswap.finance/#/exchange/swap/{address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://rpc.ftm.tools',
    ]
)

DEX = AttributeDict(
    spookyswap=BaseDEX(
        name='SpookySwap', router_address='0xF491e7B69E4244ad4002BC14e878a34207E38c29',
        factory_address='0x152eE697f2E276fA89E96742e9bB9aB1F2E61bE3', website='https://spookyswap.finance/',
    ),
    spiritswap=BaseDEX(
        name='SpiritSwap', router_address='0x16327E3FbDaCA3bcF7E38F5Af2599D2DDc33aE52',
        factory_address='0xEF45d134b73241eDa7703fa787148D9C9F4950b0', website='https://www.spiritswap.finance/',
    ),
)

FANTOM = BaseChain(
    name='Fantom Opera',
    explorer='https://ftmscan.com/',
    weth=WFTM,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=250,
    dex=DEX,
)



from ._base import *

WCRO = BaseWETH(
    name='Wrapped CRO',
    symbol='WCRO',
    address='0x5C7F8A570d578ED84E63fdFA7b1eE72dEae1AE23',
)

STABLE_COINS = BaseCoin(
    {
        'DAI': '0xF2001B145b43032AAF5Ee2884e456CCd805F677D',
        'USDC': '0xc21223249CA28397B4B6541dfFaEcC539BfF0c59',
        'USDT': '0x66e428c3f67a68878562e79A0234c1F83c208770',
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/cronos/{lp_address}',
        'dexscreener': 'https://dexscreener.com/cronos/{lp_address}',
        'Sell/Buy On MMF': 'https://mm.finance/swap?outputCurrency={address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://evm-cronos.crypto.org',  
        'https://evm.cronos.org', 
        'https://cronosrpc-1.xstaking.sg',
        'https://cronos-rpc.elk.finance'
    ]
)

DEX = AttributeDict(
    mmf=BaseDEX(
        name='Mad Meerkat Finance', router_address='0x145677FC4d9b8F19B5D56d1820c48e0443049a30',
        factory_address='0xd590cC180601AEcD6eeADD9B7f2B7611519544f4', website='https://mm.finance/swap',
    )
)

CRONOS = BaseChain(
    name='Cronos Mainnet',
    explorer='https://cronoscan.com/',
    weth=WCRO,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=25,
    dex=DEX,
)



from ._base import *

WETH = BaseWETH(
    name='Wrapped ETH',
    symbol='WETH',
    address='0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2',
)

STABLE_COINS = BaseCoin(
    {
        'USDT': '0xdAC17F958D2ee523a2206206994597C13D831ec7',
        'USDC': '0xA0b86991c6218b36c1d19D4a2e9Eb0cE3606eB48',
        'DAI': '0x6B175474E89094C44Da98b954EedeAC495271d0F',
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/ethereum/{lp_address}',
        'dexscreener': 'https://dexscreener.com/ethereum/{lp_address}',
        'dextools': 'https://www.dextools.io/app/uniswap/pair-explorer/{address}',
        'Sell/Buy On Uniswap-V2': 'https://app.uniswap.org/#/swap?outputCurrency={address}&use=V2',
    }
)

RPC = BaseRPC(
    all=[
        'https://cloudflare-eth.com',
    ]
)

DEX = AttributeDict(
    uniswap_v2=BaseDEX(
        name='Uniswap V2', router_address='0x7a250d5630B4cF539739dF2C5dAcb4c659F2488D',
        factory_address='0x5C69bEe701ef814a2B6a3EDD4B1652CB9cc5aA6f', website='https://app.uniswap.org/',
    )
)

ETH = BaseChain(
    name='Ethereum Chain',
    explorer='https://etherscan.io/',
    weth=WETH,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=1,
    dex=DEX,
)



from ._base import *

WKCS = BaseWETH(
    name='Wrapped KCS',
    symbol='WKCS',
    address='0x4446fc4eb47f2f6586f9faab68b3498f86c07521',
)

STABLE_COINS = BaseCoin(
    {
        'USDT': '0x0039f574ee5cc39bdd162e9a88e3eb1f111baf48',
        'USDC': '0x980a5afef3d17ad98635f6c5aebcbaeded3c3430'
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/kcc/{lp_address}',
        'dexscreener': 'https://dexscreener.com/kcc/{lp_address}',
        'koffeeswap': 'https://koffeeswap.exchange/#/pro',
        'Sell/Buy On KoffeeSwap': 'https://koffeeswap.exchange/#/swap?outputCurrency={address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://rpc-mainnet.kcc.network',
    ]
)

DEX = AttributeDict(
    koffeeswap=BaseDEX(
        name='KoffeeSwap', router_address='0xc0fFee0000C824D24E0F280f1e4D21152625742b',
        factory_address='0xC0fFeE00000e1439651C6aD025ea2A71ED7F3Eab', website='https://koffeeswap.exchange/',
    )
)

KCC = BaseChain(
    name='KuCoin Community Chain',
    explorer='https://explorer.kcc.io/en/',
    weth=WKCS,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=321,
    dex=DEX,
)



from ..chains import CHAINS
from ..chains import SUPPORT_CHAIN
from ..chains._base import BaseChain, BaseDEX


def assert_chain_supported(chain: str):
    '''
    chain: UPPER str, like BSC, ETH, ARBITRUM
    '''
    chain_name = chain.upper()
    assert chain_name in SUPPORT_CHAIN, f"{chain} is not supported, support = [{SUPPORT_CHAIN}]"


def get_chain(chain: str) -> BaseChain:
    assert_chain_supported(chain)
    chain_name = chain.upper()
    return CHAINS[chain_name]


def get_support_dex(chain: str) -> list:
    chain_module = get_chain(chain)
    return list(chain_module.dex.keys())


def assert_dex_supported(chain: str, dex: str):
    assert_chain_supported(chain)
    support_dex = get_support_dex(chain)
    assert dex in support_dex, f"{dex} is not supported, support = {support_dex}"


def get_dex(chain: str, dex: str) -> BaseDEX:
    assert_dex_supported(chain, dex)
    dex_name = dex.lower()
    chain_name = chain.upper()
    return CHAINS[chain_name].dex[dex_name]


def get(chain, dex=None):
    # replace "-" to "_"
    chain = chain.replace('-', '_')
    if dex is None:
        return get_chain(chain)
    dex = dex.replace('-', '_')
    return get_dex(chain, dex)


def print_all_dex():
    for name, chain in CHAINS.items():
        print(f"- {name}:")
        for j, (d_name, dex) in enumerate(chain.dex.items()):
            print(f"  - [{j + 1}] [{dex.name}]({dex.website})")



from ._base import *

WMOVR = BaseWETH(
    name='Wrapped MOVR',
    symbol='WMOVR',
    address='0x98878B06940aE243284CA214f92Bb71a2b032B8A',
)

STABLE_COINS = BaseCoin(
    {
        'USDC': '0xE3F5a90F9cb311505cd691a46596599aA1A0AD7D',
        'DAI': '0x80A16016cC4A2E6a2CACA8a4a498b1699fF0f844',
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/moonriver/{lp_address}',
        'dexscreener': 'https://dexscreener.com/moonriver/{lp_address}',
        'ChartEX': 'https://chartex.pro/dashboard',
        'Moonlit': 'https://moonlit.finance/',
        'Sell/Buy On Solarbeam': 'https://app.solarbeam.io/exchange/swap',
    }
)

RPC = BaseRPC(
    all=[
        'https://rpc.moonriver.moonbeam.network',
    ]
)

DEX = AttributeDict(
    solarbeam=BaseDEX(
        name='Solarbeam', router_address='0xAA30eF758139ae4a7f798112902Bf6d65612045f',
        factory_address='0x049581aEB6Fe262727f290165C29BDAB065a1B68', website='https://app.solarbeam.io/exchange/swap',
    )
)

MOONRIVER = BaseChain(
    name='Moonriver',
    explorer='https://moonriver.moonscan.io',
    weth=WMOVR,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=1285,
    dex=DEX,
)



from ._base import *

WAVAX = BaseWETH(
    name='Wrapped AVAX',
    symbol='WAVAX',
    address='0xB31f66AA3C1e785363F0875A1B74E27b85FD66c7',
)

# from https://snowtrace.io/tokens
STABLE_COINS = BaseCoin(
    {
        'USDT.e': '0xc7198437980c041c805A1EDcbA50c1Ce5db95118',
        'USDC.e': '0xA7D7079b0FEaD91F3e65f86E8915Cb59c1a4C664',
        'Dai.e': '0xd586E7F844cEa2F87f50152665BCbc2C279D8d70',
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/avalanche/{lp_address}',
        'ChartEX': 'https://chartex.pro/dashboard',
        'dexscreener': 'https://dexscreener.com/avalanche/{lp_address}',
        'Sell/Buy On traderjoexyz': 'https://traderjoexyz.com/#/trade',
    }
)

RPC = BaseRPC(
    all=[
        'https://api.avax.network/ext/bc/C/rpc',
    ]
)

DEX = AttributeDict(
    joe=BaseDEX(
        name='traderjoexyz', router_address='0x60ae616a2155ee3d9a68541ba4544862310933d4',
        router_abi=AVAX_JOE_ROUTER_ABI, factory_address='0x9ad6c38be94206ca50bb0d90783181662f0cfa10',
        factory_abi=AVAX_JOE_FACTORY_ABI, lp_abi=AVAX_JOE_PAIR_ABI, website='https://traderjoexyz.com/#/home',
    )
)

AVAX = BaseChain(
    name='Avalanche Network',
    explorer='https://snowtrace.io/',
    weth=WAVAX,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=43114,
    dex=DEX,
)



from ._base import *

WBNB = BaseWETH(
    name='Wrapped BNB',
    symbol='WBNB',
    address='0xae13d989daC2f0dEbFf460aC112a837C89BAa7cd',
)

STABLE_COINS = BaseCoin(
    {
        'BUSD': '0xeD24FC36d5Ee211Ea25A80239Fb8C4Cfd80f12Ee',
        'USDT': '0x337610d27c682E347C9cD60BD4b3b107C9d34dDd',
        'USDC': '0x64544969ed7EBf5f083679233325356EbE738930',
    }
)

# https://testnet.bscscan.com/address/0xaa25aa7a19f9c426e07dee59b12f944f4d9f1dd3#tokentxns
TEST_COINS = {
    'BTCB': '0x6ce8dA28E2f864420840cF74474eFf5fD80E65B8',
    'ETH': '0xd66c6B4F0be8CE5b39D52E0Fd1344c389929B378',
    'XRP': '0xa83575490D7df4E2F47b7D38ef351a2722cA45b9'
}

CHARTS = BaseChart(
    {
        'default': 'https://test-binance-chain-charts/this-is-not-real-url/{address}',
        'Sell/Buy On UNREAL SWAP': 'https://test-binance-chain-swap/this-is-not-real-url/{address}/#/swap?outputCurrency={address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://data-seed-prebsc-1-s1.binance.org:8545/',
        'https://data-seed-prebsc-2-s1.binance.org:8545/',
        'https://data-seed-prebsc-1-s2.binance.org:8545/',
        'https://data-seed-prebsc-2-s2.binance.org:8545/',
        'https://data-seed-prebsc-1-s3.binance.org:8545/',
        'https://data-seed-prebsc-2-s3.binance.org:8545/',
    ]
)

DEX = AttributeDict(
    pancake_v2_test=BaseDEX(
        name='PancakeSwap v2 (TEST)', router_address='0xD99D1c33F9fC3444f8101754aBC46c52416550D1',
        factory_address='0x6725F303b657a9451d8BA641348b6761A6CC7a17', website=''
    ),
)

BSC_TEST = BaseChain(
    name='Binance Smart Chain Testnet',
    explorer='https://testnet.bscscan.com/',
    weth=WBNB,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=97,
    dex=DEX,
)



from ._base import *

WHT = BaseWETH(
    name='Wrapped HT',
    symbol='WHT',
    address='0x5545153CCFcA01fbd7Dd11C0b23ba694D9509A6F',
)

STABLE_COINS = BaseCoin(
    {
        'USDT HECO': '0xa71edc38d189767582c38a3145b5873052c3e47a',
        'USDC HECO': '0x9362bbef4b8313a8aa9f0c9808b80577aa26b73b',
        'DAI HECO': '0x3d760a45d0887dfd89a2f5385a236b29cb46ed2a'
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/heco/{lp_address}',
        'dexscreener': 'https://dexscreener.com/heco/{lp_address}',
        'pippi': 'https://info.pippi.finance/token/{address}',  # only support
        'nut': 'https://info.nut.money/token/{address}',
        # xfarmer shold input pair address
        'xfarmer': 'https://xfarmer.net/trade.html?pair_address={address}',
        'Sell/Buy On MDEX-HECO': 'https://ht.mdex.co/#/swap?outputCurrency={address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://http-mainnet.hecochain.com',  # other
        'https://http-mainnet-node.huobichain.com',  # china
        'https://http-mainnet-node.defibox.com',
    ]
)

DEX = AttributeDict(
    mdex_heco=BaseDEX(
        name='MDEX (HECO)', router_address='0xED7d5F38C79115ca12fe6C0041abb22F0A06C300',
        factory_address='0xb0b670fc1F7724119963018DB0BfA86aDb22d941', website='https://ht.mdex.co/#/swap/',
    )
)

HECO = BaseChain(
    name='Huobi Eco Chain',
    explorer='https://hecoinfo.com/',
    weth=WHT,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=128,
    dex=DEX,
)



from ._base import *

WOKT = BaseWETH(
    name='Wrapped OKT',
    symbol='WOKT',
    address='0x8f8526dbfd6e38e3d8307702ca8469bae6c56c15',
)

STABLE_COINS = BaseCoin(
    {
        'USDT': '0x382bb369d343125bfb2117af9c149795c6c65c50',
        'USDC': '0xc946daf81b08146b1c7a8da2a851ddf2b3eaaf85',
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/oec/{lp_address}',
        'dexscreener': 'https://dexscreener.com/oec/{lp_address}',
        'Sell/Buy On CherrySwap': 'https://www.cherryswap.net/#/swap?outputCurrency={address}',
        'Sell/Buy On JSwap': 'https://app.jswap.finance/#/swap?outputCurrency={address}',
        'Sell/Buy On Pandaex': 'https://app.pandaex.org/#/swap?outputCurrency={address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://exchainrpc.okex.org',
    ]
)

DEX = AttributeDict(
    cherryswap=BaseDEX(
        name='CherrySwap', router_address='0x865bfde337C8aFBffF144Ff4C29f9404EBb22b15',
        factory_address='0x709102921812B3276A65092Fe79eDfc76c4D4AFe', website='https://www.cherryswap.net/',
    ),

    jswap=BaseDEX(
        name='JSwap', router_address='0x069A306A638ac9d3a68a6BD8BE898774C073DCb3',
        factory_address='0xd654CbF99F2907F06c88399AE123606121247D5C', website='https://app.jswap.finance/',
    ),
    pandaswap=BaseDEX(
        name='PandaSwap', router_address='0x6eC2A127057c13Dff1D04e0ae131a756aa4033d4',
        factory_address='0x328C72A24d63E3d903ba9272B95A23ec919eB7E1', website='https://app.pandaex.org/',
    )
)

OKEX = BaseChain(
    name='OKExChain',
    explorer='https://www.oklink.com/okexchain/',
    weth=WOKT,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=66,
    dex=DEX,
)



from .chain_rpc_enum import CHAIN_RPC_API
from .chain_scan_enum import CHAIN_SCAN_API
from .contract_api_enum import CHAIN_CONTRACT_API

__all__ = ['CHAIN_RPC_API', 'CHAIN_SCAN_API', 'CHAIN_CONTRACT_API']



from ._base import *

WBNB = BaseWETH(
    name='Wrapped BNB',
    symbol='WBNB',
    address='0xbb4CdB9CBd36B01bD1cBaEBF2De08d9173bc095c',
)

STABLE_COINS = BaseCoin(
    {
        'BUSD': '0xe9e7cea3dedca5984780bafc599bd69add087d56',
        'USDT': '0x55d398326f99059ff775485246999027b3197955',
        'USDC': '0x8ac76a51cc950d9822d68b83fe1ad97b32cd580d',
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://poocoin.app/tokens/{address}',
        'poocoin': 'https://poocoin.app/tokens/{address}',
        'dextools': 'https://www.dextools.io/app/pancakeswap/pair-explorer/{address}',
        'dex.guru': 'https://dex.guru/token/{address}-bsc',
        'bogged': 'https://charts.bogged.finance/?token={address}',
        'avedex': 'https://avedex.org/token/{address}',
        'dexscreener': 'https://dexscreener.com/bsc/{lp_address}',
        'Sell/Buy On Pancake-V2': 'https://exchange.pancakeswap.finance/#/swap?outputCurrency={address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://bsc-dataseed.binance.org/',
        'https://bsc-dataseed1.binance.org/',
        'https://bsc-dataseed2.binance.org/',
        'https://bsc-dataseed3.binance.org/',
        'https://bsc-dataseed4.binance.org/',
        'https://bsc-dataseed1.defibit.io/',
        'https://bsc-dataseed2.defibit.io/',
        'https://bsc-dataseed3.defibit.io/',
        'https://bsc-dataseed4.defibit.io/',
        'https://bsc-dataseed1.ninicoin.io/',
        'https://bsc-dataseed2.ninicoin.io/',
        'https://bsc-dataseed3.ninicoin.io/',
        'https://bsc-dataseed4.ninicoin.io/',
    ]
)

DEX = AttributeDict(
    pancake_v2=BaseDEX(
        name='PancakeSwap v2', router_address='0x10ED43C718714eb63d5aA57B78B54704E256024E',
        factory_address='0xcA143Ce32Fe78f1f7019d7d551a6402fC5350c73',
        website='https://exchange.pancakeswap.finance/#/swap'
    ),
    mdex_bsc=BaseDEX(
        name='MDEX (BSC)', router_address='0x7DAe51BD3E3376B8c7c4900E9107f12Be3AF1bA8',
        factory_address='0x3CD1C46068dAEa5Ebb0d3f55F6915B10648062B8', website='https://bsc.mdex.co/#/swap/'
    ),
    apeswap=BaseDEX(
        name='ApeSwap Finance', router_address='0xcF0feBd3f17CEf5b47b0cD257aCf6025c5BFf3b7',
        factory_address='0x0841BD0B734E4F5853f0dD8d7Ea041c241fb0Da6', website='https://app.apeswap.finance/swap'
    ),
    biswap=BaseDEX(
        name='BiSwap', router_address='0x3a6d8cA21D1CF76F653A67577FA0D27453350dD8',
        factory_address='0x858E3312ed3A876947EA49d572A7C42DE08af7EE', website='https://exchange.biswap.org/#/swap'
    ),
    # bakery=BaseDEX(
    #     name='BakerySwap', router_address='0xBf6527834dBB89cdC97A79FCD62E6c08B19F8ec0', router_abi=BAKERY_ROUTER_ABI, factory_address='0x8a1E9d3aEbBBd5bA2A64d3355A48dD5E9b511256', factory_abi=BAKERY_FACTORY_ABI, website='https://www.bakeryswap.org/'),
    jul=BaseDEX(
        name='Just Liquidity', router_address='0xbd67d157502A23309Db761c41965600c2Ec788b2',
        factory_address='0x553990F2CBA90272390f62C5BDb1681fFc899675', website='https://dexbeta.julswap.com/'
    )
)

BSC = BaseChain(
    name='Binance Smart Chain',
    explorer='https://bscscan.com/',
    weth=WBNB,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=56,
    dex=DEX,
)



from dataclasses import dataclass


@dataclass(frozen=True)
class CHAIN_RPC_API:
    ETH: str = 'https://cloudflare-eth.com'
    BSC: str = 'https://bsc-dataseed1.defibit.io'
    HECO: str = 'https://http-mainnet-node.huobichain.com'
    MATIC: str = 'https://rpc-mainnet.maticvigil.com'
    KCC: str = 'https://rpc-mainnet.kcc.network'
    OKEX: str = 'https://exchainrpc.okex.org'
    MOONRIVER: str = 'https://rpc.moonriver.moonbeam.network'
    FTM: str = 'https://rpc.ftm.tools'
    AVAX: str = 'https://api.avax.network/ext/bc/C/rpc'
    ARBITRUM: str = 'https://arb1.arbitrum.io/rpc'
    CRONOS: str = 'https://evm.cronos.org'



from setuptools import setup, find_packages

setup(
    name='many_abis',
    packages=find_packages(),
    version='0.1.3',
    license='MIT',
    description='A simple way to get different DEXs abis for block chains.',
    author='Yong',
    author_email='ackness8@gmail.com',
    url='https://github.com/ackness/many_abis',
    keywords=['abi', 'dex', 'block chain', 'bsc', 'eth', 'matic', 'heco', 'kcc', 'pancakeswap', 'mdex', 'quickswap',
              'uniswap', 'koffeeswap'],
    install_requires=[
        # 'requests',
        # 'web3',
    ]
)



from ._base import *

WMATIC = BaseWETH(
    name='Wrapped Matic',
    symbol='WMATIC',
    address='0x0d500b1d8e8ef31e21c99d1db9a6444d3adf1270',
)

STABLE_COINS = BaseCoin(
    {
        'USDC': '0x2791bca1f2de4661ed88a30c99a7a9449aa84174',
        'DAI': '0x8f3cf7ad23cd3cadbd9735aff958023239c6a063',
        'PUSD': '0x9af3b7dc29d3c4b1a5731408b6a9656fa7ac3b72'
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/polygon/{address}',
        'dexscreener': 'https://dexscreener.com/polygon/{address}',
        'poocoin': 'https://polygon.poocoin.app/tokens/{address}',
        'kek': 'https://kek.tools/t/{address}',
        'quickcharts': 'https://quickchart.app/token/{address}',
        'polychart': 'https://app.polychart.io/explorer/polygon/{address}',
        'dex.guru': 'https://dex.guru/token/{address}-polygon',
        'Sell/Buy On QuickSwap': 'https://quickswap.exchange/#/swap?outputCurrency={address}',
    }
)

RPC = BaseRPC(
    all=[
        'https://rpc-mainnet.matic.network',
        'https://rpc-mainnet.maticvigil.com',
        'https://rpc-mainnet.matic.quiknode.pro',
        'https://matic-mainnet.chainstacklabs.com',
        'https://matic-mainnet-full-rpc.bwarelabs.com',
        'https://matic-mainnet-archive-rpc.bwarelabs.com',
    ]
)

DEX = AttributeDict(
    quickswap=BaseDEX(
        name='QuickSwap', router_address='0xa5E0829CaCEd8fFDD4De3c43696c57F7D7A678ff',
        factory_address='0x5757371414417b8C6CAad45bAeF941aBc7d3Ab32', website='https://quickswap.exchange/',
    )
)

POLYGON = BaseChain(
    name='Polygon PoS Chain',
    explorer='https://polygonscan.com/',
    weth=WMATIC,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=137,
    dex=DEX,
)



from ._base import AttributeDict
from .arbitrum import ARBITRUM
from .avax import AVAX
from .bsc import BSC
from .bsc_test import BSC_TEST
from .cronos import CRONOS
from .eth import ETH
from .fantom import FANTOM
from .heco import HECO
from .kcc import KCC
from .moonriver import MOONRIVER
from .okex import OKEX
from .polygon import POLYGON


SUPPORT_CHAIN = ['BSC', 'ETH', 'AVAX', 'FANTOM', 'POLYGON', 'CRONOS',
                 'HECO', 'KCC', 'OKEX', 'MOONRIVER', 'ARBITRUM', 'BSC_TEST']
__all__ = ['SUPPORT_CHAIN', 'BSC', 'ETH', 'AVAX', 'FANTOM', 'CRONOS',
           'POLYGON', 'HECO', 'KCC', 'OKEX', 'MOONRIVER', 'BSC_TEST', 'ARBITRUM', 'CHAINS']

CHAINS = AttributeDict(
    **{name: eval(name) for name in SUPPORT_CHAIN}
)



from .chains import *
from .chains._assets import STANDARD_ERC20_TOKEN_ABI as ERC20_ABI
from .chains._assets import WETH_ABI, UNISWAP_V2_ROUTER_ABI, UNISWAP_V2_FACTORY_ABI, UNISWAP_V2_PAIR_ABI
from .utils import *
from .utils.enums.chain_rpc_enum import CHAIN_RPC_API
from .utils.enums.chain_scan_enum import CHAIN_SCAN_API
from .utils.enums.contract_api_enum import CHAIN_CONTRACT_API



from ._base import *

WETH = BaseWETH(
    name='Wrapped ETH',
    symbol='WETH',
    address='0x82aF49447D8a07e3bd95BD0d56f35241523fBab1',
)

STABLE_COINS = BaseCoin(
    {
        'USDT': '0xfd086bc7cd5c481dcc9c85ebe478a1c0b69fcbb9',
        'USDC': '0xff970a61a04b1ca14834a43f5de4533ebddb5cc8'
    }
)

CHARTS = BaseChart(
    {
        'default': 'https://dexscreener.com/arbitrum/{lp_address}',
        'dexscreener': 'https://dexscreener.com/arbitrum/{lp_address}',
        'SushiSwap': 'https://app.sushi.com/en/swap',
        'Sell/Buy On SushiSwap': 'https://app.sushi.com/en/swap',
    }
)

RPC = BaseRPC(
    all=[
        'https://arb1.arbitrum.io/rpc',
    ]
)

DEX = AttributeDict(
    suhiswap=BaseDEX(
        name='SushiSwap', router_address='0x1b02dA8Cb0d097eB8D57A175b88c7D8b47997506',
        factory_address='0xc35DADB65012eC5796536bD9864eD8773aBc74C4', website='https://app.sushi.com/en/swap',
    )
)

ARBITRUM = BaseChain(
    name='Arbitrum One',
    explorer='https://arbiscan.io',
    weth=WETH,
    coins=STABLE_COINS,
    charts=CHARTS,
    rpcs=RPC,
    chain_id=42161,
    dex=DEX,
)



from typing import Any
from web3 import Web3
from ._assets import *


class AttributeDict(dict):
    """
    A dictionary that can get attribute (x.y) access, including nested dicts.
    """

    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)
        self.__parse_address()

    def __setattr__(self, __name: str, __value: Any) -> None:
        if type(__value) == dict:
            __value = AttributeDict(__value)
            for __key in __value:
                __value.__setattr__(__key, __value[__key])
        self.__setitem__(__name, __value)

    def __parse_address(self, ):
        for k, v in self.items():
            if bool(Web3.isAddress(v)):
                self[k] = Web3.toChecksumAddress(v)

    __getattr__ = dict.get
    __delattr__ = dict.__delitem__


class BaseWETH(AttributeDict):
    def __init__(
            self,
            name: str = None, symbol: str = None,
            address: str = None, abi: dict = WETH_ABI
    ):
        super().__init__(name=name, symbol=symbol, address=address, abi=abi)


class BaseCoin(AttributeDict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class BaseChart(AttributeDict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class BaseRPC(AttributeDict):
    def __init__(self, *args, **kwargs):
        self.all = None
        super().__init__(*args, **kwargs)

    def __repr__(self):
        return f"All rpcs: {self.all}"


class BaseDEX(AttributeDict):
    def __init__(
            self,
            name: str = None,
            router_address: str = None, router_abi: dict = UNISWAP_V2_ROUTER_ABI,
            factory_address: str = None, factory_abi: dict = UNISWAP_V2_FACTORY_ABI,
            lp_abi: dict = UNISWAP_V2_PAIR_ABI,
            website: str = None,
    ):
        super().__init__(
            name=name,
            router_address=router_address,
            router_abi=router_abi,
            factory_address=factory_address,
            factory_abi=factory_abi,
            lp_abi=lp_abi,
            website=website,
        )

    def __repr__(self):
        return f'DEX Name: {self.name}, Website: {self.website}, Router: {self.router_address}, ' \
               f'Factory: {self.factory_address}'


class BaseChain(AttributeDict):
    def __init__(
            self,
            name: str = None, explorer: str = None,
            weth: BaseWETH = None, coins: BaseCoin = None,
            charts: BaseChart = None, rpcs: BaseRPC = None,
            chain_id: int = 0, dex: AttributeDict = None,
    ):
        super().__init__(
            name=name,
            explorer=explorer,
            weth=weth,
            coins=coins,
            charts=charts,
            rpcs=rpcs,
            chain_id=chain_id,
            dex=dex,
        )

    def __repr__(self):
        return f'Chain Name: {self.name}, Explorer: {self.explorer}, DEX: {self.dex.keys()}'
